@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -25,6 +25,10 @@
 #ifndef _LIBPRELUDE_IDMEF_CLASS_H
 #define _LIBPRELUDE_IDMEF_CLASS_H
 
+#ifdef __cplusplus
+ extern "C" {
+#endif
+
 typedef int idmef_class_id_t;
 typedef int idmef_class_child_id_t;
 
@@ -35,8 +39,16 @@ typedef int idmef_class_child_id_t;
  */
 prelude_bool_t idmef_class_is_child_list(idmef_class_id_t classid, idmef_class_child_id_t child);
 
+prelude_bool_t idmef_class_is_child_keyed_list(idmef_class_id_t classid, idmef_class_child_id_t child);
+
+prelude_bool_t idmef_class_is_child_union_member(idmef_class_id_t classid, idmef_class_child_id_t child);
+
+int idmef_class_get_child_union_id(idmef_class_id_t classid, idmef_class_child_id_t child);
+
 idmef_class_id_t idmef_class_get_child_class(idmef_class_id_t classid, idmef_class_child_id_t child);
 
+size_t idmef_class_get_child_count(idmef_class_id_t classid);
+
 idmef_value_type_id_t idmef_class_get_child_value_type(idmef_class_id_t classid, idmef_class_child_id_t child);
 
 
@@ -55,6 +67,8 @@ int idmef_class_get_child(void *ptr, idmef_class_id_t classid, idmef_class_child
 
 int idmef_class_new_child(void *ptr, idmef_class_id_t classid, idmef_class_child_id_t child, int n, void **childptr);
 
+int idmef_class_destroy_child(void *ptr, idmef_class_id_t classid, idmef_class_child_id_t child, int n);
+
 
 /*
  *
@@ -82,6 +96,16 @@ int idmef_class_compare(idmef_class_id_t classid, const void *c1, const void *c2
 
 int idmef_class_ref(idmef_class_id_t classid, void *obj);
 
+int idmef_class_print(idmef_class_id_t classid, void *obj, prelude_io_t *fd);
+
+int idmef_class_print_json(idmef_class_id_t classid, void *obj, prelude_io_t *fd);
+
 int idmef_class_destroy(idmef_class_id_t classid, void *obj);
 
+prelude_bool_t idmef_class_is_listed(idmef_class_id_t classid);
+
+#ifdef __cplusplus
+ }
+#endif
+
 #endif /* _LIBPRELUDE_IDMEF_CLASS_H */
