@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -25,10 +25,10 @@
 #include "libmissing.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <gnutls/gnutls.h>
-#include <gcrypt.h>
 
 #include "glthread/thread.h"
 #include "glthread/lock.h"
@@ -58,59 +58,6 @@ static void tls_log_func(int level, const char *data)
         prelude_log(PRELUDE_LOG_INFO, "%s", data);
 }
 
-static int gcry_prelude_mutex_init(void **retval)
-{
-        int ret;
-        gl_lock_t *lock;
-
-        *retval = lock = malloc(sizeof(*lock));
-        if ( ! lock )
-                return ENOMEM;
-
-        ret = glthread_lock_init(lock);
-        if ( ret < 0 )
-                free(lock);
-
-        return ret;
-}
-
-
-static int gcry_prelude_mutex_destroy(void **lock)
-{
-        return glthread_lock_destroy(*lock);
-}
-
-
-
-static int gcry_prelude_mutex_lock(void **lock)
-{
-        return glthread_lock_lock((gl_lock_t *) *lock);
-}
-
-
-static int gcry_prelude_mutex_unlock(void **lock)
-{
-        return glthread_lock_unlock((gl_lock_t *) *lock);
-}
-
-
-static struct gcry_thread_cbs gcry_threads_prelude = {
-        GCRY_THREAD_OPTION_USER,
-        NULL,
-        gcry_prelude_mutex_init,
-        gcry_prelude_mutex_destroy,
-        gcry_prelude_mutex_lock,
-        gcry_prelude_mutex_unlock,
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        NULL
-};
-
 
 static void slice_arguments(int *argc, char **argv)
 {
@@ -226,11 +173,6 @@ int prelude_init(int *argc, char **argv)
 
         slice_arguments(argc, argv);
 
-        ret = gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_prelude);
-        if ( ret < 0 )
-                return prelude_error_verbose(PRELUDE_ERROR_TLS,
-                                             "gcrypt initialization failed: %s", gcry_strerror(ret));
-
         ret = gnutls_global_init();
         if ( ret < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_TLS,
